//
//  BCIDAMASConstraintMaker.h
//  Masonry
//
//  Created by Jonas Budelmann on 20/07/13.
//  Copyright (c) 2013 cloudling. All rights reserved.
//

#import "BCIDAMASConstraint.h"
#import "BCIDAMASUtilities.h"

typedef NS_OPTIONS(NSInteger, bcMASAttribute) {
    bcMASAttributeLeft = 1 << NSLayoutAttributeLeft,
    bcMASAttributeRight = 1 << NSLayoutAttributeRight,
    bcMASAttributeTop = 1 << NSLayoutAttributeTop,
    bcMASAttributeBottom = 1 << NSLayoutAttributeBottom,
    bcMASAttributeLeading = 1 << NSLayoutAttributeLeading,
    bcMASAttributeTrailing = 1 << NSLayoutAttributeTrailing,
    bcMASAttributeWidth = 1 << NSLayoutAttributeWidth,
    bcMASAttributeHeight = 1 << NSLayoutAttributeHeight,
    bcMASAttributeCenterX = 1 << NSLayoutAttributeCenterX,
    bcMASAttributeCenterY = 1 << NSLayoutAttributeCenterY,
    bcMASAttributeBaseline = 1 << NSLayoutAttributeBaseline,

    bcMASAttributeFirstBaseline = 1 << NSLayoutAttributeFirstBaseline,
    bcMASAttributeLastBaseline = 1 << NSLayoutAttributeLastBaseline,
    
#if TARGET_OS_IPHONE || TARGET_OS_TV
    
    bcMASAttributeLeftMargin = 1 << NSLayoutAttributeLeftMargin,
    bcMASAttributeRightMargin = 1 << NSLayoutAttributeRightMargin,
    bcMASAttributeTopMargin = 1 << NSLayoutAttributeTopMargin,
    bcMASAttributeBottomMargin = 1 << NSLayoutAttributeBottomMargin,
    bcMASAttributeLeadingMargin = 1 << NSLayoutAttributeLeadingMargin,
    bcMASAttributeTrailingMargin = 1 << NSLayoutAttributeTrailingMargin,
    bcMASAttributeCenterXWithinMargins = 1 << NSLayoutAttributeCenterXWithinMargins,
    bcMASAttributeCenterYWithinMargins = 1 << NSLayoutAttributeCenterYWithinMargins,

#endif
    
};

/**
 *  Provides factory methods for creating BCIDAMASConstraints.
 *  Constraints are collected until they are ready to be installed
 *
 */
@interface BCIDAMASConstraintMaker : NSObject

/**
 *	The following properties return a new MASViewConstraint
 *  with the first item set to the makers associated view and the appropriate MASViewAttribute
 */
@property (nonatomic, strong, readonly) BCIDAMASConstraint *left;
@property (nonatomic, strong, readonly) BCIDAMASConstraint *top;
@property (nonatomic, strong, readonly) BCIDAMASConstraint *right;
@property (nonatomic, strong, readonly) BCIDAMASConstraint *bottom;
@property (nonatomic, strong, readonly) BCIDAMASConstraint *leading;
@property (nonatomic, strong, readonly) BCIDAMASConstraint *trailing;
@property (nonatomic, strong, readonly) BCIDAMASConstraint *width;
@property (nonatomic, strong, readonly) BCIDAMASConstraint *height;
@property (nonatomic, strong, readonly) BCIDAMASConstraint *centerX;
@property (nonatomic, strong, readonly) BCIDAMASConstraint *centerY;
@property (nonatomic, strong, readonly) BCIDAMASConstraint *baseline;

@property (nonatomic, strong, readonly) BCIDAMASConstraint *firstBaseline;
@property (nonatomic, strong, readonly) BCIDAMASConstraint *lastBaseline;

#if TARGET_OS_IPHONE || TARGET_OS_TV

@property (nonatomic, strong, readonly) BCIDAMASConstraint *leftMargin;
@property (nonatomic, strong, readonly) BCIDAMASConstraint *rightMargin;
@property (nonatomic, strong, readonly) BCIDAMASConstraint *topMargin;
@property (nonatomic, strong, readonly) BCIDAMASConstraint *bottomMargin;
@property (nonatomic, strong, readonly) BCIDAMASConstraint *leadingMargin;
@property (nonatomic, strong, readonly) BCIDAMASConstraint *trailingMargin;
@property (nonatomic, strong, readonly) BCIDAMASConstraint *centerXWithinMargins;
@property (nonatomic, strong, readonly) BCIDAMASConstraint *centerYWithinMargins;

#endif

/**
 *  Returns a block which creates a new BCIDAMASCompositeConstraint with the first item set
 *  to the makers associated view and children corresponding to the set bits in the
 *  bcMASAttribute parameter. Combine multiple attributes via binary-or.
 */
@property (nonatomic, strong, readonly) BCIDAMASConstraint *(^attributes)(bcMASAttribute attrs);

/**
 *	Creates a BCIDAMASCompositeConstraint with type BCIDAMASCompositeConstraintTypeEdges
 *  which generates the appropriate MASViewConstraint children (top, left, bottom, right)
 *  with the first item set to the makers associated view
 */
@property (nonatomic, strong, readonly) BCIDAMASConstraint *edges;

/**
 *	Creates a BCIDAMASCompositeConstraint with type BCIDAMASCompositeConstraintTypeSize
 *  which generates the appropriate MASViewConstraint children (width, height)
 *  with the first item set to the makers associated view
 */
@property (nonatomic, strong, readonly) BCIDAMASConstraint *size;

/**
 *	Creates a BCIDAMASCompositeConstraint with type BCIDAMASCompositeConstraintTypeCenter
 *  which generates the appropriate MASViewConstraint children (centerX, centerY)
 *  with the first item set to the makers associated view
 */
@property (nonatomic, strong, readonly) BCIDAMASConstraint *center;

/**
 *  Whether or not to check for an existing constraint instead of adding constraint
 */
@property (nonatomic, assign) BOOL updateExisting;

/**
 *  Whether or not to remove existing constraints prior to installing
 */
@property (nonatomic, assign) BOOL removeExisting;

/**
 *	initialises the maker with a default view
 *
 *	@param	view	any BCIDAMASConstraint are created with this view as the first item
 *
 *	@return	a new BCIDAMASConstraintMaker
 */
- (id)initWithView:(bcMAS_VIEW *)view;

/**
 *	Calls install method on any BCIDAMASConstraints which have been created by this maker
 *
 *	@return	an array of all the installed BCIDAMASConstraints
 */
- (NSArray *)install;

- (BCIDAMASConstraint * (^)(dispatch_block_t))group;

@end
