//
//  UIView+BCIDAMASAdditions.h
//  Masonry
//
//  Created by Jonas Budelmann on 20/07/13.
//  Copyright (c) 2013 cloudling. All rights reserved.
//

#import "BCIDAMASUtilities.h"
#import "BCIDAMASConstraintMaker.h"
#import "BCIDAMASViewAttribute.h"

/**
 *	Provides constraint maker block
 *  and convience methods for creating MASViewAttribute which are view + NSLayoutAttribute pairs
 */
@interface bcMAS_VIEW (BCIDAMASAdditions)

/**
 *	following properties return a new MASViewAttribute with current view and appropriate NSLayoutAttribute
 */
@property (nonatomic, strong, readonly) BCIDAMASViewAttribute *mas_left;
@property (nonatomic, strong, readonly) BCIDAMASViewAttribute *mas_top;
@property (nonatomic, strong, readonly) BCIDAMASViewAttribute *mas_right;
@property (nonatomic, strong, readonly) BCIDAMASViewAttribute *mas_bottom;
@property (nonatomic, strong, readonly) BCIDAMASViewAttribute *mas_leading;
@property (nonatomic, strong, readonly) BCIDAMASViewAttribute *mas_trailing;
@property (nonatomic, strong, readonly) BCIDAMASViewAttribute *mas_width;
@property (nonatomic, strong, readonly) BCIDAMASViewAttribute *mas_height;
@property (nonatomic, strong, readonly) BCIDAMASViewAttribute *mas_centerX;
@property (nonatomic, strong, readonly) BCIDAMASViewAttribute *mas_centerY;
@property (nonatomic, strong, readonly) BCIDAMASViewAttribute *mas_baseline;
@property (nonatomic, strong, readonly) BCIDAMASViewAttribute *(^mas_attribute)(NSLayoutAttribute attr);

@property (nonatomic, strong, readonly) BCIDAMASViewAttribute *mas_firstBaseline;
@property (nonatomic, strong, readonly) BCIDAMASViewAttribute *mas_lastBaseline;

#if TARGET_OS_IPHONE || TARGET_OS_TV

@property (nonatomic, strong, readonly) BCIDAMASViewAttribute *mas_leftMargin;
@property (nonatomic, strong, readonly) BCIDAMASViewAttribute *mas_rightMargin;
@property (nonatomic, strong, readonly) BCIDAMASViewAttribute *mas_topMargin;
@property (nonatomic, strong, readonly) BCIDAMASViewAttribute *mas_bottomMargin;
@property (nonatomic, strong, readonly) BCIDAMASViewAttribute *mas_leadingMargin;
@property (nonatomic, strong, readonly) BCIDAMASViewAttribute *mas_trailingMargin;
@property (nonatomic, strong, readonly) BCIDAMASViewAttribute *mas_centerXWithinMargins;
@property (nonatomic, strong, readonly) BCIDAMASViewAttribute *mas_centerYWithinMargins;

@property (nonatomic, strong, readonly) BCIDAMASViewAttribute *mas_safeAreaLayoutGuide NS_AVAILABLE_IOS(11.0);
@property (nonatomic, strong, readonly) BCIDAMASViewAttribute *mas_safeAreaLayoutGuideLeading NS_AVAILABLE_IOS(11.0);
@property (nonatomic, strong, readonly) BCIDAMASViewAttribute *mas_safeAreaLayoutGuideTrailing NS_AVAILABLE_IOS(11.0);
@property (nonatomic, strong, readonly) BCIDAMASViewAttribute *mas_safeAreaLayoutGuideLeft NS_AVAILABLE_IOS(11.0);
@property (nonatomic, strong, readonly) BCIDAMASViewAttribute *mas_safeAreaLayoutGuideRight NS_AVAILABLE_IOS(11.0);
@property (nonatomic, strong, readonly) BCIDAMASViewAttribute *mas_safeAreaLayoutGuideTop NS_AVAILABLE_IOS(11.0);
@property (nonatomic, strong, readonly) BCIDAMASViewAttribute *mas_safeAreaLayoutGuideBottom NS_AVAILABLE_IOS(11.0);
@property (nonatomic, strong, readonly) BCIDAMASViewAttribute *mas_safeAreaLayoutGuideWidth NS_AVAILABLE_IOS(11.0);
@property (nonatomic, strong, readonly) BCIDAMASViewAttribute *mas_safeAreaLayoutGuideHeight NS_AVAILABLE_IOS(11.0);
@property (nonatomic, strong, readonly) BCIDAMASViewAttribute *mas_safeAreaLayoutGuideCenterX NS_AVAILABLE_IOS(11.0);
@property (nonatomic, strong, readonly) BCIDAMASViewAttribute *mas_safeAreaLayoutGuideCenterY NS_AVAILABLE_IOS(11.0);

#endif

/**
 *	a key to associate with this view
 */
@property (nonatomic, strong) id mas_key;

/**
 *	Finds the closest common superview between this view and another view
 *
 *	@param	view	other view
 *
 *	@return	returns nil if common superview could not be found
 */
- (instancetype)mas_closestCommonSuperview:(bcMAS_VIEW *)view;

/**
 *  Creates a BCIDAMASConstraintMaker with the callee view.
 *  Any constraints defined are added to the view or the appropriate superview once the block has finished executing
 *
 *  @param block scope within which you can build up the constraints which you wish to apply to the view.
 *
 *  @return Array of created BCIDAMASConstraints
 */
- (NSArray *)mas_makeConstraints:(void(NS_NOESCAPE ^)(BCIDAMASConstraintMaker *make))block;

/**
 *  Creates a BCIDAMASConstraintMaker with the callee view.
 *  Any constraints defined are added to the view or the appropriate superview once the block has finished executing.
 *  If an existing constraint exists then it will be updated instead.
 *
 *  @param block scope within which you can build up the constraints which you wish to apply to the view.
 *
 *  @return Array of created/updated BCIDAMASConstraints
 */
- (NSArray *)mas_updateConstraints:(void(NS_NOESCAPE ^)(BCIDAMASConstraintMaker *make))block;

/**
 *  Creates a BCIDAMASConstraintMaker with the callee view.
 *  Any constraints defined are added to the view or the appropriate superview once the block has finished executing.
 *  All constraints previously installed for the view will be removed.
 *
 *  @param block scope within which you can build up the constraints which you wish to apply to the view.
 *
 *  @return Array of created/updated BCIDAMASConstraints
 */
- (NSArray *)mas_remakeConstraints:(void(NS_NOESCAPE ^)(BCIDAMASConstraintMaker *make))block;

@end
